unit ICComponent;

(* Start Header *)

(* File:         ICComponent.p
 * Generated by: 1.0d6
 * For:          IC 2.0
 * On:           Wednesday, 6 May 1998, 23:12:40
 * 
 * This file is part of the Internet Configuration system and
 * is placed in the public domain for the benefit of all.
 *)

(* End Header *)

interface

	uses
		Components;

{$main}
	function Main (var params: ComponentParameters; storage: Ptr): ComponentResult;

implementation

	uses
		Types,
		Files,
		Memory, 
		Errors, 
		ToolUtils, 
		AppleTalk, 
		Aliases, 
		Packages, 
		GestaltEqu, 
		Resources, 
		Dialogs,
		AppleEvents, 
		Folders, 
		Traps,

		{$ifc not undefined MWERKS}
			PascalA4,
		{$endc}

		InternetConfig, 

		ICDebug,
		ICInstrumentation,
		ICCommonSubs,
		ICComponentResources,
		ICSystemGlobals,
		ICCore,
		ICURLHandling,
		ICMapping,
		ICProfiles;

	(* ***** Component Manager Routines ***** *)
	
	(* Routines that are generally implemented by components. *)

	function ICCICanDo (globals: ComponentGlobalsPtr; selector: integer): ComponentResult;
		(* Handle the Component Manager CanDo request. We can do a standard set
			of Component Manager defined selectors, and all the selectors defined
			in ICComponentSelectors.p.
		*)
	begin
		{$unused globals}
		case selector of
			kComponentVersionSelect,
				kComponentCanDoSelect,
					kComponentOpenSelect,
					kComponentCloseSelect,
					kComponentRegisterSelect,
					kComponentTargetSelect,
					kICCFirstSelector..kICCLastSelector: 
				ICCICanDo := 1;
			otherwise
				ICCICanDo := 0;
		end; (* case *)
	end; (* ICCICanDo *)

	{$ifc icInstrumentation}
		const
			gOpenCount : longint = 0;
			gInstrumentationActive : Boolean = false;
	{$endc}
	
	function ICCIOpen (globals: ComponentGlobalsPtr; self: ComponentInstance): ComponentResult;
		(*	Handle the Component Manager Open request.  We do very little here, basically
			using it as an opportunity to create our globals.  Most of the real initialisation
			is done in the Start routine.
			
			The globals are created in the current heap zone, not in the system heap.  This
			is very important, because, when a heap zone is destroyed (typically when an
			application quits), the Component Manager automagically cleans any instances of
			any components whose storage parameter points into that heap.  By creating our
			globals in the current heap, we ensure that a) we get cleaned up by Component
			Manager if we're opened by an application, and b) if an extension wants to talk
			to IC, it can do so simply by doing a SetZone(SystemZone()) before calling
			ICStart/ICCStart.
		*)
		var
			err: ComponentResult;
	begin
		(* Create our globals. *)
		globals := ComponentGlobalsPtr(NewPtrClear(sizeof(ComponentGlobals)));
		err := MemError;

		(* Initialise some specified fields and tell the Component Manager about our globals. *)
		if err = noErr then begin
			globals^.self := self;
			globals^.target := self;
			SetComponentInstanceStorage(self, Handle(globals));
		end; (* if *)
		
		{$ifc icInstrumentation}
			ICAssert(gOpenCount >= 0);
			gOpenCount := gOpenCount + 1;
			if gOpenCount = 1 then begin
				DebugStr('Starting InstrumentationLib');
				if InstInitialize68K = noErr then begin
					gInstrumentationActive := true;
					InitICMappingInstrumentation;
				end; (* if *)
			end; (* if *)
		{$endc}

		ICCIOpen := noErr;
	end; (* ICCIOpen *)

	function ICCIClose (globals: ComponentGlobalsPtr; self: ComponentInstance): ComponentResult;
		(*	Handle the Component Manager Close request.  We presume that the client
			has correctly called Stop, so all we have to do is dispose of our globals
			and leave.  We also Stop the instance if it hasn't been already stopped by
			the client.
		*)
		var
			err: ComponentResult;
			junk : ICError;
	begin
		{$unused self}
		err := noErr;
		if globals <> nil then begin
			if globals^.haveStarted then begin
				ComponentWatchPoint(globals, kICDebugSignificantEventsMask, 'Close is forcing a Stop', 0);
				junk := ICCIStop(globals);
				globals^.haveStarted := false;
			end; (* if *)
			DisposePtr(Ptr(globals));
			ICAssert(MemError() = noErr);
		end; (* if *)

		{$ifc icInstrumentation}
			gOpenCount := gOpenCount - 1;
			if gOpenCount = 0 then begin
				if gInstrumentationActive then begin
					DebugStr('Stopping InstrumentationLib');
					junk := InstTerminate68K;
					ICAssert( junk = noErr );
					gInstrumentationActive := false;
				end; (* if *)
			end; (* if *)
			ICAssert(gOpenCount >= 0);
		{$endc}

		ICCIClose := err;
	end; (* ICCIClose *)
	
	function ICCIRegister(globals: ComponentGlobalsPtr): ComponentResult;
		var
			err : ComponentResult;
			gestaltValue : longint;
	begin
		err := noErr;
		if not TrapAvailable(_Gestalt) | 
				(Gestalt(gestaltSystemVersion, gestaltValue) <> noErr) | 
				(gestaltValue < $0700) then begin
			err := unimpErr;
		end; (* if *)
		if err = noErr then begin
			if FindICSystemGlobals(globals) = nil then begin
				err := RegisterSystemGlobals(globals);
			end; (* if *)
		end; (* if *)
		if err <> noErr then begin
			err := 1;					// Inside Mac says the Register routine should return true to not be registered
		end; (* if *)
		ICCIRegister := err;
	end; (* ICCIRegister *)

	function ICCITarget (globals: ComponentGlobalsPtr; newTarget: ComponentInstance): ComponentResult;
		(* Handle the Component Manager Target.  We change our current target
			variable to reflect the new target.
		*)
	begin
		globals^.target := newTarget;
		ICCITarget := noErr;
	end; (* ICCITarget *)

(* ----- Component Entry Point ----- *)

	function Main (var params: ComponentParameters; storage: Ptr): ComponentResult;
		(* The main component entry point.  It's just a standard component dispatcher. *)

		{$ifc icDebug}

			procedure ComponentEntryExitDebug(entry : Boolean; result : ComponentResult);
				var
					sysGlobals : ICSystemGlobalsPtr;
			begin
				if storage <> nil then begin
					sysGlobals := FindICSystemGlobals(ComponentGlobalsPtr(storage));
					if sysGlobals <> nil then begin
						if entry then begin
							if band(sysGlobals^.debugFlags, kICDebugComponentEntryMask) <> 0 then begin
								DebugStr("IC Component Entry " + SelectorToStr(params.what));
							end; (* if *)
						end else begin
							if band(sysGlobals^.debugFlags, kICDebugComponentExitMask) <> 0 then begin
								DebugStr("IC Component Exit  " + SelectorToStr(params.what) + " " + DecStr(result));
							end; (* if *)
						end; (* if *)
					end; (* if *)
				end; (* if *)
			end; (* ComponentEntryExitDebug *)

		{$endc}

		var
			oldA4 : longint;
			oldA5 : longint;
			proc: ProcPtr;
			result : ComponentResult;
	begin
		(*	If we're being compiled under Metrowerks, bring in the correct A4 world.
			Under both Metrowerks and MPW, set up A5 to point to the current application's
			A5 world.  IC doesn't use A5 based globals, except insofar as system API
			routines use them.  So we don't need to create our own A5 world.  But we
			do need to make sure our A5 world is correct, because we call system
			routines (like ModalDialog, which calls QuickDraw, and Unique1ID, which
			calls Random) that need A5 to point to some valid A5 world.  A5 may not
			be pointing to a valid A5 world in all possible in which we're called
			(the case that triggered this change was IC running under Mac OS 8 with
			the NetLaunchURL override component installed, where A5 was nil) so
			we make sure it's the case here.
		*)
		{$ifc not undefined MWERKS} oldA4 := SetCurrentA4; {$endc}
		oldA5 := SetCurrentA5;

		{$ifc icDebug}
			ComponentEntryExitDebug(true, 0);
		{$endc}
		
		(* Setup proc based on which routine is to be called. *)
		
		proc := nil;
		case params.what of
			(* Component Manager stuff *)
			kComponentVersionSelect: 
				result := kICComponentInterfaceVersion + 0;
			kComponentCanDoSelect: 
				proc := @ICCICanDo;
			kComponentOpenSelect: 
				proc := @ICCIOpen;
			(* Component Manager stuff *)
			kComponentCloseSelect: 
				proc := @ICCIClose;
			kComponentRegisterSelect:
				proc := @ICCIRegister;
			kComponentTargetSelect: 
				proc := @ICCITarget;
			(* this component type stuff *)
(* Start ICCSwitch.p *)

			kICCStart: 
			    proc := @ICCIStart;
			kICCStop: 
			    proc := @ICCIStop;
			kICCFindConfigFile: 
			    proc := @ICCIFindConfigFile;
			kICCFindUserConfigFile: 
			    proc := @ICCIFindUserConfigFile;
			kICCGeneralFindConfigFile: 
			    proc := @ICCIGeneralFindConfigFile;
			kICCChooseConfig: 
			    proc := @ICCIChooseConfig;
			kICCChooseNewConfig: 
			    proc := @ICCIChooseNewConfig;
			kICCGetConfigName: 
			    proc := @ICCIGetConfigName;
			kICCGetConfigReference: 
			    proc := @ICCIGetConfigReference;
			kICCSetConfigReference: 
			    proc := @ICCISetConfigReference;
			kICCSpecifyConfigFile: 
			    proc := @ICCISpecifyConfigFile;
			kICCRefreshCaches: 
			    proc := @ICCIRefreshCaches;
			kICCGetSeed: 
			    proc := @ICCIGetSeed;
			kICCGetPerm: 
			    proc := @ICCIGetPerm;
			kICCDefaultFileName: 
			    proc := @ICCIDefaultFileName;
			kICCBegin: 
			    proc := @ICCIBegin;
			kICCGetPref: 
			    proc := @ICCIGetPref;
			kICCSetPref: 
			    proc := @ICCISetPref;
			kICCFindPrefHandle: 
			    proc := @ICCIFindPrefHandle;
			kICCGetPrefHandle: 
			    proc := @ICCIGetPrefHandle;
			kICCSetPrefHandle: 
			    proc := @ICCISetPrefHandle;
			kICCCountPref: 
			    proc := @ICCICountPref;
			kICCGetIndPref: 
			    proc := @ICCIGetIndPref;
			kICCDeletePref: 
			    proc := @ICCIDeletePref;
			kICCEnd: 
			    proc := @ICCIEnd;
			kICCEditPreferences: 
			    proc := @ICCIEditPreferences;
			kICCParseURL: 
			    proc := @ICCIParseURL;
			kICCLaunchURL: 
			    proc := @ICCILaunchURL;
			kICCMapFilename: 
			    proc := @ICCIMapFilename;
			kICCMapTypeCreator: 
			    proc := @ICCIMapTypeCreator;
			kICCMapEntriesFilename: 
			    proc := @ICCIMapEntriesFilename;
			kICCMapEntriesTypeCreator: 
			    proc := @ICCIMapEntriesTypeCreator;
			kICCCountMapEntries: 
			    proc := @ICCICountMapEntries;
			kICCGetIndMapEntry: 
			    proc := @ICCIGetIndMapEntry;
			kICCGetMapEntry: 
			    proc := @ICCIGetMapEntry;
			kICCSetMapEntry: 
			    proc := @ICCISetMapEntry;
			kICCDeleteMapEntry: 
			    proc := @ICCIDeleteMapEntry;
			kICCAddMapEntry: 
			    proc := @ICCIAddMapEntry;
			kICCGetCurrentProfile: 
			    proc := @ICCIGetCurrentProfile;
			kICCSetCurrentProfile: 
			    proc := @ICCISetCurrentProfile;
			kICCCountProfiles: 
			    proc := @ICCICountProfiles;
			kICCGetIndProfile: 
			    proc := @ICCIGetIndProfile;
			kICCGetProfileName: 
			    proc := @ICCIGetProfileName;
			kICCSetProfileName: 
			    proc := @ICCISetProfileName;
			kICCAddProfile: 
			    proc := @ICCIAddProfile;
			kICCDeleteProfile: 
			    proc := @ICCIDeleteProfile;
			kICCRequiresInterruptSafe: 
			    proc := @ICCIRequiresInterruptSafe;
			kICCGetMappingInterruptSafe: 
			    proc := @ICCIGetMappingInterruptSafe;
			kICCGetSeedInterruptSafe: 
			    proc := @ICCIGetSeedInterruptSafe;

(* End ICCSwitch.p *)
			otherwise
				result := badComponentSelector;
		end; (* case *)

		(* If we've got a routine to call, call it.  Note that the default case
			above has set result up to return the appropriate error code if
			proc is nil.
		*)
		if proc <> nil then begin
			result := CallComponentFunctionWithStorage(Handle(storage), params, proc);
		end; (* if *)

		(* Clean up A5 and A4. *)

		{$ifc icDebug}
			ComponentEntryExitDebug(false, result);
		{$endc}
		
		oldA5 := SetA5(oldA5);
		{$ifc not undefined MWERKS} oldA4 := SetA4(oldA4); {$endc}
		Main := result;
	end; (* Main *)

end. (* ICComponent *)